package com.persagy.apm.common.model.entity;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AccessLevel;
import lombok.Data;
import lombok.Getter;
import lombok.Setter;

import java.io.Serializable;
import java.util.Date;

/**
 * 实体基类RequiredParamsStorage
 * @author Charlie Yu
 * @version 1.0 2021-03-04
 */
@Data
@ApiModel
public abstract class BaseEntity<T> extends Model implements IBaseEntity,Serializable,Cloneable{

	/** 序列id */
	private static final long serialVersionUID = -3231114188611235386L;

	/** 实体属性主键id */
	public final static String PROP_ID = "id";

	/** 实体属性版本 */
	public final static String PROP_TS = "ts";

	/** 实体属性实体状态 */
	public final static String PROP_VALID = "valid";

	/** hash码 */
	@Getter(AccessLevel.NONE)
	@Setter(AccessLevel.NONE)
	@TableField(exist=false)
	private int hashCode = Integer.MIN_VALUE;

	/** 实体类必须有的属性 */

	@TableId(type = IdType.ASSIGN_ID)
	@ApiModelProperty("主键")
	protected String id;

	/** 时间戳 */
	@ApiModelProperty("乐观锁")
	protected Date ts;

	/** 实体状态 - 有效标识 */
	@TableField(fill = FieldFill.INSERT)
	@ApiModelProperty(value = "删除标记", notes = "com.persagy.apm.common.constant.enums.ValidEnum")
	protected Integer valid;

	@Override
	public boolean equals (Object obj) {
		if (null == obj) {
            return false;
        }
		if (!(obj instanceof BaseEntity)) {
			return false;
		} else {
			BaseEntity entity = (BaseEntity) obj;
			if (null == this.getId() || null == entity.getId()) {
				return false;
			} else {
				return (this.getId().equals(entity.getId()));
			}
		}
	}

	@Override
	public int hashCode () {
		if (Integer.MIN_VALUE != this.hashCode) {
			return this.hashCode;
		}
		if (null == this.getId()) {
			return super.hashCode();
		}
		String hashStr = this.getClass().getName() + ":" + this.getId().hashCode();
		this.hashCode = hashStr.hashCode();
		return this.hashCode;
	}

	@Override
	public int compareTo (IBaseEntity obj) {
		if (obj.hashCode() > hashCode()) {
			return 1;
		} else if (obj.hashCode() < hashCode()) {
			return -1;
		} else {
			return 0;
		}
	}

	@Override
	public Object clone() throws CloneNotSupportedException {
		return super.clone();
	}

}
