package com.persagy.apm.common.utils;

import cn.hutool.http.useragent.OS;
import cn.hutool.http.useragent.UserAgent;
import cn.hutool.http.useragent.UserAgentUtil;
import com.persagy.apm.common.constant.CommonConstant;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.List;

/***
 * Description: 文件处理公共类
 * @author : lijie
 * @date :2021/4/28 16:38
 * Update By lijie 2021/4/28 16:38
 */
@Slf4j
public class FileUtil {

    /***
     * Description: 获取资源文件方法
     * @param configFilePath : 配置文件路径
     * @param classpathFilePath : 本地路径
     * @return : org.springframework.core.io.Resource[]
     * @author : lijie
     * @date :2021/3/1 14:25
     * Update By lijie 2021/3/1 14:25
     */
    public static Resource[] getResource(String configFilePath, String classpathFilePath) throws IOException {
        ResourcePatternResolver resolver = new PathMatchingResourcePatternResolver();
        Resource[] resolverResources = resolver.getResources(configFilePath);
        if (resolverResources.length > 0) {
            boolean exist = true;
            for (Resource resource : resolverResources) {
                if (!resource.exists()) {
                    exist = false;
                    break;
                }
            }
            if(exist) {
                return resolverResources;
            }
        }
        return resolver.getResources(classpathFilePath);
    }

    /***
     * Description: 写错误提示语到文件中
     * @param errorList : 错误提示语列表
     * @param response : 响应体对象
     * @author : lijie
     * @date :2021/3/4 15:33
     * Update By lijie 2021/3/4 15:33
     */
    public static void writeErrorContentToResponse(List<String> errorList,
                                            HttpServletResponse response,
                                            HttpServletRequest request,String errorFileName){
        try {
            ServletOutputStream outputStream = response.getOutputStream();
            String lineSeparator = getSystemLineSeparator(request);
            for (String error : errorList) {
                outputStream.write(error.getBytes());
                outputStream.write(lineSeparator.getBytes());
            }
            // 为文件命名
            response.setHeader(CommonConstant.REPONSE_HEAD_CONTENT_DISPOSITION,
                    "attachment;filename=" + URLEncoder.encode(errorFileName, "UTF-8"));
            response.addHeader(CommonConstant.REPONSE_HEAD_CONTENT_TYPE, "text/plain");
            response.addHeader(CommonConstant.UPLOAD_FLAG,"0");
            response.addHeader(CommonConstant.REPONSE_HEAD_ACCESS_CONTROL_EXPOSE_HEADERS,CommonConstant.UPLOAD_FLAG);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
    }

    /***
     * Description: 获得请求的换行符
     * @param request : 请求体
     * @return : void
     * @author : lijie
     * @date :2021/3/13 10:57
     * Update By lijie 2021/3/13 10:57
     */
    private static String getSystemLineSeparator(HttpServletRequest request) {
        String userAgent = request.getHeader(CommonConstant.OS_TYPE);
        if (StringUtils.isBlank(userAgent)){
            return System.lineSeparator();
        }
        UserAgent agent = UserAgentUtil.parse(userAgent);
        OS os = agent.getOs();
        if (null==os){
            return System.lineSeparator();
        }
        String osStr = agent.getOs().toString().toLowerCase();
        if (StringUtils.isBlank(osStr)){
            return System.lineSeparator();
        }
        if (osStr.contains(CommonConstant.SYSTEM_TYPE_WINDOWS)){
            return "\r\n";
        }
        if (osStr.contains(CommonConstant.SYSTEM_TYPE_LINUX)){
            return "\r";
        }
        if (osStr.contains(CommonConstant.SYSTEM_TYPE_MAC)){
            return "\n";
        }
        return System.lineSeparator();
    }

}
