package com.persagy.apm.translate.impl;

import com.persagy.apm.common.helper.SpringHelper;
import com.persagy.apm.translate.dao.ITranslateDao;
import com.persagy.apm.translate.extension.MapPairResultHandler;
import com.persagy.apm.translate.extension.MapResultHandler;
import com.persagy.apm.translate.model.ITransWrapper;
import com.persagy.apm.translate.model.TransMeta;
import com.persagy.apm.translate.model.TransType;
import com.persagy.apm.translate.service.ITranslateBatchHandle;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.Map;


/**
 * 基于db的数据翻译
 * @author Charlie Yu
 * @Date 2021-03-03
 */
@Service(TransType.DB)
@Scope("prototype")
public class DBTranslateHandleImpl implements ITranslateBatchHandle {

    /**
     * 翻译元数据
     */
    private TransMeta meta;

    /**
     * 数据库dao
     */
    private ITranslateDao dao = null;

    /**
     * 查询语句
     */
    private String sql = null;

    /**
     * 目标字段
     */
    private String[] targetFields = null;

    /**
     * 结果字段
     */
    private String[] resultFields = null;

    /**
     * 取得翻译元数据
     *
     * @return 翻译元数据
     */
    @Override
    public TransMeta getMeta() {
        return meta;
    }

    /**
     * 设置翻译元数据
     *
     * @param meta 翻译元数据
     */
    @Override
    public void setMeta(TransMeta meta) {
        this.meta = meta;
    }

    @Override
    public ITransWrapper trans(ITransWrapper data) {
        String parameter = (String) data.getValue(meta.getParameter());
        if (StringUtils.isEmpty(parameter)) {
            //参数值为空，则目标值也为空
            targetFields = meta.getTarget().split(",");
            for (int i = 0; i < targetFields.length; i++) {
                data.setValue(targetFields[i], null);
            }
            return data;
        }
        if (dao == null) {
            dao = SpringHelper.getBean(ITranslateDao.class);
        }
        if (sql == null) {
            String table = meta.getSource();
            String column = meta.getResult();
            String con = meta.getCondition();
            targetFields = meta.getTarget().split(",");
            resultFields = meta.getResultFields();
            sql = "select " + column + " from " + table + " where " + con + "=#{parameter}";
        }
        Map<String, Object> map = dao.findTransBySql(sql, parameter);
        for (int i = 0; i < resultFields.length; i++) {
            Object r = map == null ? null : map.get(resultFields[i]);
            if (r == null && map != null) {
                r = map.get(resultFields[i].toUpperCase());
            }
            if (r != null) {
                data.setValue(targetFields[i], r.toString());
            } else {
                data.setValue(targetFields[i], r);
            }

        }
        return data;
    }

    @Override
    public Map<String, Object> batchTrans(List<String> parameter) {
        if (parameter == null || parameter.size() == 0) {
            return null;
        }
        if (dao == null) {
            dao = SpringHelper.getBean(ITranslateDao.class);
        }
        String table = meta.getSource();
        String column = meta.getResult();
        String con = meta.getCondition();
        String key = meta.getMapKey();
        int idx = key.indexOf(" as ");
        if (idx > 0) {
            key = key.substring(idx + 4).trim();
        }
        String selectPart = meta.getMapKey() + "," + column;
        String sql = "select " + selectPart + " from " + table + " where " + con + " in";
        String[] fields = meta.getResultFields();
        String[] ids = parameter.stream().distinct().toArray(String[]::new);
        if (fields == null || fields.length == 0) {
            return null;
        } else if (fields.length > 1) {
            MapResultHandler handler = new MapResultHandler(key);
            dao.findBatchTransBySql(sql, Arrays.asList(ids), handler);
            return handler.getResult();
        } else {
            MapPairResultHandler handler = new MapPairResultHandler(key, fields[0]);
            dao.findBatchTransBySql(sql, Arrays.asList(ids), handler);
            return handler.getResult();
        }
    }

}
